/*!
  * Bootstrap alert.js v4.3.1 (https://getbootstrap.com/)
  * Copyright 2011-2019 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
  */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('jquery'), require('./util.js')) :
	typeof define === 'function' && define.amd ? define(['jquery', './util.js'], factory) :
	(global.Alert = factory(global.jQuery,global.Util));
}(this, (function ($,Util) { 'use strict';

$ = $ && $.hasOwnProperty('default') ? $['default'] : $;
Util = Util && Util.hasOwnProperty('default') ? Util['default'] : Util;

/**
 * --------------------------------------------------------------------------
 * Bootstrap (v4.3.1): alert.js
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * --------------------------------------------------------------------------
 */

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */

var NAME                = 'alert';
var VERSION             = '4.3.1';
var DATA_KEY            = 'bs.alert';
var EVENT_KEY           = "." + DATA_KEY;
var DATA_API_KEY        = '.data-api';
var JQUERY_NO_CONFLICT  = $.fn[NAME];

var Selector = {
  DISMISS : '[data-dismiss="alert"]'
};

var Event = {
  CLOSE          : ("close" + EVENT_KEY),
  CLOSED         : ("closed" + EVENT_KEY),
  CLICK_DATA_API : ("click" + EVENT_KEY + DATA_API_KEY)
};

var ClassName = {
  ALERT : 'alert',
  FADE  : 'fade',
  SHOW  : 'show'
};

/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Alert = function Alert(element) {
  this._element = element;
};

var staticAccessors = { VERSION: { configurable: true } };

// Getters

staticAccessors.VERSION.get = function () {
  return VERSION
};

// Public

Alert.prototype.close = function close (element) {
  var rootElement = this._element;
  if (element) {
    rootElement = this._getRootElement(element);
  }

  var customEvent = this._triggerCloseEvent(rootElement);

  if (customEvent.isDefaultPrevented()) {
    return
  }

  this._removeElement(rootElement);
};

Alert.prototype.dispose = function dispose () {
  $.removeData(this._element, DATA_KEY);
  this._element = null;
};

// Private

Alert.prototype._getRootElement = function _getRootElement (element) {
  var selector = Util.getSelectorFromElement(element);
  var parent   = false;

  if (selector) {
    parent = document.querySelector(selector);
  }

  if (!parent) {
    parent = $(element).closest(("." + (ClassName.ALERT)))[0];
  }

  return parent
};

Alert.prototype._triggerCloseEvent = function _triggerCloseEvent (element) {
  var closeEvent = $.Event(Event.CLOSE);

  $(element).trigger(closeEvent);
  return closeEvent
};

Alert.prototype._removeElement = function _removeElement (element) {
    var this$1 = this;

  $(element).removeClass(ClassName.SHOW);

  if (!$(element).hasClass(ClassName.FADE)) {
    this._destroyElement(element);
    return
  }

  var transitionDuration = Util.getTransitionDurationFromElement(element);

  $(element)
    .one(Util.TRANSITION_END, function (event) { return this$1._destroyElement(element, event); })
    .emulateTransitionEnd(transitionDuration);
};

Alert.prototype._destroyElement = function _destroyElement (element) {
  $(element)
    .detach()
    .trigger(Event.CLOSED)
    .remove();
};

// Static

Alert._jQueryInterface = function _jQueryInterface (config) {
  return this.each(function () {
    var $element = $(this);
    var data     = $element.data(DATA_KEY);

    if (!data) {
      data = new Alert(this);
      $element.data(DATA_KEY, data);
    }

    if (config === 'close') {
      data[config](this);
    }
  })
};

Alert._handleDismiss = function _handleDismiss (alertInstance) {
  return function (event) {
    if (event) {
      event.preventDefault();
    }

    alertInstance.close(this);
  }
};

Object.defineProperties( Alert, staticAccessors );

/**
 * ------------------------------------------------------------------------
 * Data Api implementation
 * ------------------------------------------------------------------------
 */

$(document).on(
  Event.CLICK_DATA_API,
  Selector.DISMISS,
  Alert._handleDismiss(new Alert())
);

/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 */

$.fn[NAME]             = Alert._jQueryInterface;
$.fn[NAME].Constructor = Alert;
$.fn[NAME].noConflict  = function () {
  $.fn[NAME] = JQUERY_NO_CONFLICT;
  return Alert._jQueryInterface
};

return Alert;

})));
//# sourceMappingURL=alert.js.map
