/*!
  * Bootstrap carousel.js v4.3.1 (https://getbootstrap.com/)
  * Copyright 2011-2019 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
  */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('jquery'), require('./util.js')) :
	typeof define === 'function' && define.amd ? define(['jquery', './util.js'], factory) :
	(global.Carousel = factory(global.jQuery,global.Util));
}(this, (function ($,Util) { 'use strict';

$ = $ && $.hasOwnProperty('default') ? $['default'] : $;
Util = Util && Util.hasOwnProperty('default') ? Util['default'] : Util;

/**
 * --------------------------------------------------------------------------
 * Bootstrap (v4.3.1): carousel.js
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * --------------------------------------------------------------------------
 */

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */

var NAME                   = 'carousel';
var VERSION                = '4.3.1';
var DATA_KEY               = 'bs.carousel';
var EVENT_KEY              = "." + DATA_KEY;
var DATA_API_KEY           = '.data-api';
var JQUERY_NO_CONFLICT     = $.fn[NAME];
var ARROW_LEFT_KEYCODE     = 37; // KeyboardEvent.which value for left arrow key
var ARROW_RIGHT_KEYCODE    = 39; // KeyboardEvent.which value for right arrow key
var TOUCHEVENT_COMPAT_WAIT = 500; // Time for mouse compat events to fire after touch
var SWIPE_THRESHOLD        = 40;

var Default = {
  interval : 5000,
  keyboard : true,
  slide    : false,
  pause    : 'hover',
  wrap     : true,
  touch    : true
};

var DefaultType = {
  interval : '(number|boolean)',
  keyboard : 'boolean',
  slide    : '(boolean|string)',
  pause    : '(string|boolean)',
  wrap     : 'boolean',
  touch    : 'boolean'
};

var Direction = {
  NEXT     : 'next',
  PREV     : 'prev',
  LEFT     : 'left',
  RIGHT    : 'right'
};

var Event = {
  SLIDE          : ("slide" + EVENT_KEY),
  SLID           : ("slid" + EVENT_KEY),
  KEYDOWN        : ("keydown" + EVENT_KEY),
  MOUSEENTER     : ("mouseenter" + EVENT_KEY),
  MOUSELEAVE     : ("mouseleave" + EVENT_KEY),
  TOUCHSTART     : ("touchstart" + EVENT_KEY),
  TOUCHMOVE      : ("touchmove" + EVENT_KEY),
  TOUCHEND       : ("touchend" + EVENT_KEY),
  POINTERDOWN    : ("pointerdown" + EVENT_KEY),
  POINTERUP      : ("pointerup" + EVENT_KEY),
  DRAG_START     : ("dragstart" + EVENT_KEY),
  LOAD_DATA_API  : ("load" + EVENT_KEY + DATA_API_KEY),
  CLICK_DATA_API : ("click" + EVENT_KEY + DATA_API_KEY)
};

var ClassName = {
  CAROUSEL      : 'carousel',
  ACTIVE        : 'active',
  SLIDE         : 'slide',
  RIGHT         : 'carousel-item-right',
  LEFT          : 'carousel-item-left',
  NEXT          : 'carousel-item-next',
  PREV          : 'carousel-item-prev',
  ITEM          : 'carousel-item',
  POINTER_EVENT : 'pointer-event'
};

var Selector = {
  ACTIVE      : '.active',
  ACTIVE_ITEM : '.active.carousel-item',
  ITEM        : '.carousel-item',
  ITEM_IMG    : '.carousel-item img',
  NEXT_PREV   : '.carousel-item-next, .carousel-item-prev',
  INDICATORS  : '.carousel-indicators',
  DATA_SLIDE  : '[data-slide], [data-slide-to]',
  DATA_RIDE   : '[data-ride="carousel"]'
};

var PointerType = {
  TOUCH : 'touch',
  PEN   : 'pen'
};

/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */
var Carousel = function Carousel(element, config) {
  this._items       = null;
  this._interval    = null;
  this._activeElement = null;
  this._isPaused    = false;
  this._isSliding   = false;
  this.touchTimeout = null;
  this.touchStartX  = 0;
  this.touchDeltaX  = 0;

  this._config          = this._getConfig(config);
  this._element         = element;
  this._indicatorsElement = this._element.querySelector(Selector.INDICATORS);
  this._touchSupported  = 'ontouchstart' in document.documentElement || navigator.maxTouchPoints > 0;
  this._pointerEvent    = Boolean(window.PointerEvent || window.MSPointerEvent);

  this._addEventListeners();
};

var staticAccessors = { VERSION: { configurable: true },Default: { configurable: true } };

// Getters

staticAccessors.VERSION.get = function () {
  return VERSION
};

staticAccessors.Default.get = function () {
  return Default
};

// Public

Carousel.prototype.next = function next () {
  if (!this._isSliding) {
    this._slide(Direction.NEXT);
  }
};

Carousel.prototype.nextWhenVisible = function nextWhenVisible () {
  // Don't call next when the page isn't visible
  // or the carousel or its parent isn't visible
  if (!document.hidden &&
    ($(this._element).is(':visible') && $(this._element).css('visibility') !== 'hidden')) {
    this.next();
  }
};

Carousel.prototype.prev = function prev () {
  if (!this._isSliding) {
    this._slide(Direction.PREV);
  }
};

Carousel.prototype.pause = function pause (event) {
  if (!event) {
    this._isPaused = true;
  }

  if (this._element.querySelector(Selector.NEXT_PREV)) {
    Util.triggerTransitionEnd(this._element);
    this.cycle(true);
  }

  clearInterval(this._interval);
  this._interval = null;
};

Carousel.prototype.cycle = function cycle (event) {
  if (!event) {
    this._isPaused = false;
  }

  if (this._interval) {
    clearInterval(this._interval);
    this._interval = null;
  }

  if (this._config.interval && !this._isPaused) {
    this._interval = setInterval(
      (document.visibilityState ? this.nextWhenVisible : this.next).bind(this),
      this._config.interval
    );
  }
};

Carousel.prototype.to = function to (index) {
    var this$1 = this;

  this._activeElement = this._element.querySelector(Selector.ACTIVE_ITEM);

  var activeIndex = this._getItemIndex(this._activeElement);

  if (index > this._items.length - 1 || index < 0) {
    return
  }

  if (this._isSliding) {
    $(this._element).one(Event.SLID, function () { return this$1.to(index); });
    return
  }

  if (activeIndex === index) {
    this.pause();
    this.cycle();
    return
  }

  var direction = index > activeIndex
    ? Direction.NEXT
    : Direction.PREV;

  this._slide(direction, this._items[index]);
};

Carousel.prototype.dispose = function dispose () {
  $(this._element).off(EVENT_KEY);
  $.removeData(this._element, DATA_KEY);

  this._items           = null;
  this._config          = null;
  this._element         = null;
  this._interval        = null;
  this._isPaused        = null;
  this._isSliding       = null;
  this._activeElement   = null;
  this._indicatorsElement = null;
};

// Private

Carousel.prototype._getConfig = function _getConfig (config) {
  config = Object.assign({}, Default,
    config);
  Util.typeCheckConfig(NAME, config, DefaultType);
  return config
};

Carousel.prototype._handleSwipe = function _handleSwipe () {
  var absDeltax = Math.abs(this.touchDeltaX);

  if (absDeltax <= SWIPE_THRESHOLD) {
    return
  }

  var direction = absDeltax / this.touchDeltaX;

  // swipe left
  if (direction > 0) {
    this.prev();
  }

  // swipe right
  if (direction < 0) {
    this.next();
  }
};

Carousel.prototype._addEventListeners = function _addEventListeners () {
    var this$1 = this;

  if (this._config.keyboard) {
    $(this._element)
      .on(Event.KEYDOWN, function (event) { return this$1._keydown(event); });
  }

  if (this._config.pause === 'hover') {
    $(this._element)
      .on(Event.MOUSEENTER, function (event) { return this$1.pause(event); })
      .on(Event.MOUSELEAVE, function (event) { return this$1.cycle(event); });
  }

  if (this._config.touch) {
    this._addTouchEventListeners();
  }
};

Carousel.prototype._addTouchEventListeners = function _addTouchEventListeners () {
    var this$1 = this;

  if (!this._touchSupported) {
    return
  }

  var start = function (event) {
    if (this$1._pointerEvent && PointerType[event.originalEvent.pointerType.toUpperCase()]) {
      this$1.touchStartX = event.originalEvent.clientX;
    } else if (!this$1._pointerEvent) {
      this$1.touchStartX = event.originalEvent.touches[0].clientX;
    }
  };

  var move = function (event) {
    // ensure swiping with one touch and not pinching
    if (event.originalEvent.touches && event.originalEvent.touches.length > 1) {
      this$1.touchDeltaX = 0;
    } else {
      this$1.touchDeltaX = event.originalEvent.touches[0].clientX - this$1.touchStartX;
    }
  };

  var end = function (event) {
    if (this$1._pointerEvent && PointerType[event.originalEvent.pointerType.toUpperCase()]) {
      this$1.touchDeltaX = event.originalEvent.clientX - this$1.touchStartX;
    }

    this$1._handleSwipe();
    if (this$1._config.pause === 'hover') {
      // If it's a touch-enabled device, mouseenter/leave are fired as
      // part of the mouse compatibility events on first tap - the carousel
      // would stop cycling until user tapped out of it;
      // here, we listen for touchend, explicitly pause the carousel
      // (as if it's the second time we tap on it, mouseenter compat event
      // is NOT fired) and after a timeout (to allow for mouse compatibility
      // events to fire) we explicitly restart cycling

      this$1.pause();
      if (this$1.touchTimeout) {
        clearTimeout(this$1.touchTimeout);
      }
      this$1.touchTimeout = setTimeout(function (event) { return this$1.cycle(event); }, TOUCHEVENT_COMPAT_WAIT + this$1._config.interval);
    }
  };

  $(this._element.querySelectorAll(Selector.ITEM_IMG)).on(Event.DRAG_START, function (e) { return e.preventDefault(); });
  if (this._pointerEvent) {
    $(this._element).on(Event.POINTERDOWN, function (event) { return start(event); });
    $(this._element).on(Event.POINTERUP, function (event) { return end(event); });

    this._element.classList.add(ClassName.POINTER_EVENT);
  } else {
    $(this._element).on(Event.TOUCHSTART, function (event) { return start(event); });
    $(this._element).on(Event.TOUCHMOVE, function (event) { return move(event); });
    $(this._element).on(Event.TOUCHEND, function (event) { return end(event); });
  }
};

Carousel.prototype._keydown = function _keydown (event) {
  if (/input|textarea/i.test(event.target.tagName)) {
    return
  }

  switch (event.which) {
    case ARROW_LEFT_KEYCODE:
      event.preventDefault();
      this.prev();
      break
    case ARROW_RIGHT_KEYCODE:
      event.preventDefault();
      this.next();
      break
    default:
  }
};

Carousel.prototype._getItemIndex = function _getItemIndex (element) {
  this._items = element && element.parentNode
    ? [].slice.call(element.parentNode.querySelectorAll(Selector.ITEM))
    : [];
  return this._items.indexOf(element)
};

Carousel.prototype._getItemByDirection = function _getItemByDirection (direction, activeElement) {
  var isNextDirection = direction === Direction.NEXT;
  var isPrevDirection = direction === Direction.PREV;
  var activeIndex   = this._getItemIndex(activeElement);
  var lastItemIndex = this._items.length - 1;
  var isGoingToWrap = isPrevDirection && activeIndex === 0 ||
                          isNextDirection && activeIndex === lastItemIndex;

  if (isGoingToWrap && !this._config.wrap) {
    return activeElement
  }

  var delta   = direction === Direction.PREV ? -1 : 1;
  var itemIndex = (activeIndex + delta) % this._items.length;

  return itemIndex === -1
    ? this._items[this._items.length - 1] : this._items[itemIndex]
};

Carousel.prototype._triggerSlideEvent = function _triggerSlideEvent (relatedTarget, eventDirectionName) {
  var targetIndex = this._getItemIndex(relatedTarget);
  var fromIndex = this._getItemIndex(this._element.querySelector(Selector.ACTIVE_ITEM));
  var slideEvent = $.Event(Event.SLIDE, {
    relatedTarget: relatedTarget,
    direction: eventDirectionName,
    from: fromIndex,
    to: targetIndex
  });

  $(this._element).trigger(slideEvent);

  return slideEvent
};

Carousel.prototype._setActiveIndicatorElement = function _setActiveIndicatorElement (element) {
  if (this._indicatorsElement) {
    var indicators = [].slice.call(this._indicatorsElement.querySelectorAll(Selector.ACTIVE));
    $(indicators)
      .removeClass(ClassName.ACTIVE);

    var nextIndicator = this._indicatorsElement.children[
      this._getItemIndex(element)
    ];

    if (nextIndicator) {
      $(nextIndicator).addClass(ClassName.ACTIVE);
    }
  }
};

Carousel.prototype._slide = function _slide (direction, element) {
    var this$1 = this;

  var activeElement = this._element.querySelector(Selector.ACTIVE_ITEM);
  var activeElementIndex = this._getItemIndex(activeElement);
  var nextElement = element || activeElement &&
    this._getItemByDirection(direction, activeElement);
  var nextElementIndex = this._getItemIndex(nextElement);
  var isCycling = Boolean(this._interval);

  var directionalClassName;
  var orderClassName;
  var eventDirectionName;

  if (direction === Direction.NEXT) {
    directionalClassName = ClassName.LEFT;
    orderClassName = ClassName.NEXT;
    eventDirectionName = Direction.LEFT;
  } else {
    directionalClassName = ClassName.RIGHT;
    orderClassName = ClassName.PREV;
    eventDirectionName = Direction.RIGHT;
  }

  if (nextElement && $(nextElement).hasClass(ClassName.ACTIVE)) {
    this._isSliding = false;
    return
  }

  var slideEvent = this._triggerSlideEvent(nextElement, eventDirectionName);
  if (slideEvent.isDefaultPrevented()) {
    return
  }

  if (!activeElement || !nextElement) {
    // Some weirdness is happening, so we bail
    return
  }

  this._isSliding = true;

  if (isCycling) {
    this.pause();
  }

  this._setActiveIndicatorElement(nextElement);

  var slidEvent = $.Event(Event.SLID, {
    relatedTarget: nextElement,
    direction: eventDirectionName,
    from: activeElementIndex,
    to: nextElementIndex
  });

  if ($(this._element).hasClass(ClassName.SLIDE)) {
    $(nextElement).addClass(orderClassName);

    Util.reflow(nextElement);

    $(activeElement).addClass(directionalClassName);
    $(nextElement).addClass(directionalClassName);

    var nextElementInterval = parseInt(nextElement.getAttribute('data-interval'), 10);
    if (nextElementInterval) {
      this._config.defaultInterval = this._config.defaultInterval || this._config.interval;
      this._config.interval = nextElementInterval;
    } else {
      this._config.interval = this._config.defaultInterval || this._config.interval;
    }

    var transitionDuration = Util.getTransitionDurationFromElement(activeElement);

    $(activeElement)
      .one(Util.TRANSITION_END, function () {
        $(nextElement)
          .removeClass((directionalClassName + " " + orderClassName))
          .addClass(ClassName.ACTIVE);

        $(activeElement).removeClass(((ClassName.ACTIVE) + " " + orderClassName + " " + directionalClassName));

        this$1._isSliding = false;

        setTimeout(function () { return $(this$1._element).trigger(slidEvent); }, 0);
      })
      .emulateTransitionEnd(transitionDuration);
  } else {
    $(activeElement).removeClass(ClassName.ACTIVE);
    $(nextElement).addClass(ClassName.ACTIVE);

    this._isSliding = false;
    $(this._element).trigger(slidEvent);
  }

  if (isCycling) {
    this.cycle();
  }
};

// Static

Carousel._jQueryInterface = function _jQueryInterface (config) {
  return this.each(function () {
    var data = $(this).data(DATA_KEY);
    var _config = Object.assign({}, Default,
      $(this).data());

    if (typeof config === 'object') {
      _config = Object.assign({}, _config,
        config);
    }

    var action = typeof config === 'string' ? config : _config.slide;

    if (!data) {
      data = new Carousel(this, _config);
      $(this).data(DATA_KEY, data);
    }

    if (typeof config === 'number') {
      data.to(config);
    } else if (typeof action === 'string') {
      if (typeof data[action] === 'undefined') {
        throw new TypeError(("No method named \"" + action + "\""))
      }
      data[action]();
    } else if (_config.interval && _config.ride) {
      data.pause();
      data.cycle();
    }
  })
};

Carousel._dataApiClickHandler = function _dataApiClickHandler (event) {
  var selector = Util.getSelectorFromElement(this);

  if (!selector) {
    return
  }

  var target = $(selector)[0];

  if (!target || !$(target).hasClass(ClassName.CAROUSEL)) {
    return
  }

  var config = Object.assign({}, $(target).data(),
    $(this).data());
  var slideIndex = this.getAttribute('data-slide-to');

  if (slideIndex) {
    config.interval = false;
  }

  Carousel._jQueryInterface.call($(target), config);

  if (slideIndex) {
    $(target).data(DATA_KEY).to(slideIndex);
  }

  event.preventDefault();
};

Object.defineProperties( Carousel, staticAccessors );

/**
 * ------------------------------------------------------------------------
 * Data Api implementation
 * ------------------------------------------------------------------------
 */

$(document)
  .on(Event.CLICK_DATA_API, Selector.DATA_SLIDE, Carousel._dataApiClickHandler);

$(window).on(Event.LOAD_DATA_API, function () {
  var carousels = [].slice.call(document.querySelectorAll(Selector.DATA_RIDE));
  for (var i = 0, len = carousels.length; i < len; i++) {
    var $carousel = $(carousels[i]);
    Carousel._jQueryInterface.call($carousel, $carousel.data());
  }
});

/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 */

$.fn[NAME] = Carousel._jQueryInterface;
$.fn[NAME].Constructor = Carousel;
$.fn[NAME].noConflict = function () {
  $.fn[NAME] = JQUERY_NO_CONFLICT;
  return Carousel._jQueryInterface
};

return Carousel;

})));
//# sourceMappingURL=carousel.js.map
