/*!
  * Bootstrap popover.js v4.3.1 (https://getbootstrap.com/)
  * Copyright 2011-2019 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
  */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('jquery'), require('./tooltip.js')) :
	typeof define === 'function' && define.amd ? define(['jquery', './tooltip.js'], factory) :
	(global.Popover = factory(global.jQuery,global.Tooltip));
}(this, (function ($,Tooltip) { 'use strict';

$ = $ && $.hasOwnProperty('default') ? $['default'] : $;
Tooltip = Tooltip && Tooltip.hasOwnProperty('default') ? Tooltip['default'] : Tooltip;

/**
 * --------------------------------------------------------------------------
 * Bootstrap (v4.3.1): popover.js
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * --------------------------------------------------------------------------
 */

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */

var NAME                = 'popover';
var VERSION             = '4.3.1';
var DATA_KEY            = 'bs.popover';
var EVENT_KEY           = "." + DATA_KEY;
var JQUERY_NO_CONFLICT  = $.fn[NAME];
var CLASS_PREFIX        = 'bs-popover';
var BSCLS_PREFIX_REGEX  = new RegExp(("(^|\\s)" + CLASS_PREFIX + "\\S+"), 'g');

var Default = Object.assign({}, Tooltip.Default,
  {placement : 'right',
  trigger   : 'click',
  content   : '',
  template  : '<div class="popover" role="tooltip">' +
              '<div class="arrow"></div>' +
              '<h3 class="popover-header"></h3>' +
              '<div class="popover-body"></div></div>'});

var DefaultType = Object.assign({}, Tooltip.DefaultType,
  {content : '(string|element|function)'});

var ClassName = {
  FADE : 'fade',
  SHOW : 'show'
};

var Selector = {
  TITLE   : '.popover-header',
  CONTENT : '.popover-body'
};

var Event = {
  HIDE       : ("hide" + EVENT_KEY),
  HIDDEN     : ("hidden" + EVENT_KEY),
  SHOW       : ("show" + EVENT_KEY),
  SHOWN      : ("shown" + EVENT_KEY),
  INSERTED   : ("inserted" + EVENT_KEY),
  CLICK      : ("click" + EVENT_KEY),
  FOCUSIN    : ("focusin" + EVENT_KEY),
  FOCUSOUT   : ("focusout" + EVENT_KEY),
  MOUSEENTER : ("mouseenter" + EVENT_KEY),
  MOUSELEAVE : ("mouseleave" + EVENT_KEY)
};

/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Popover = (function (Tooltip$$1) {
  function Popover () {
    Tooltip$$1.apply(this, arguments);
  }

  if ( Tooltip$$1 ) Popover.__proto__ = Tooltip$$1;
  Popover.prototype = Object.create( Tooltip$$1 && Tooltip$$1.prototype );
  Popover.prototype.constructor = Popover;

  var staticAccessors = { VERSION: { configurable: true },Default: { configurable: true },NAME: { configurable: true },DATA_KEY: { configurable: true },Event: { configurable: true },EVENT_KEY: { configurable: true },DefaultType: { configurable: true } };

  staticAccessors.VERSION.get = function () {
    return VERSION
  };

  staticAccessors.Default.get = function () {
    return Default
  };

  staticAccessors.NAME.get = function () {
    return NAME
  };

  staticAccessors.DATA_KEY.get = function () {
    return DATA_KEY
  };

  staticAccessors.Event.get = function () {
    return Event
  };

  staticAccessors.EVENT_KEY.get = function () {
    return EVENT_KEY
  };

  staticAccessors.DefaultType.get = function () {
    return DefaultType
  };

  // Overrides

  Popover.prototype.isWithContent = function isWithContent () {
    return this.getTitle() || this._getContent()
  };

  Popover.prototype.addAttachmentClass = function addAttachmentClass (attachment) {
    $(this.getTipElement()).addClass((CLASS_PREFIX + "-" + attachment));
  };

  Popover.prototype.getTipElement = function getTipElement () {
    this.tip = this.tip || $(this.config.template)[0];
    return this.tip
  };

  Popover.prototype.setContent = function setContent () {
    var $tip = $(this.getTipElement());

    // We use append for html objects to maintain js events
    this.setElementContent($tip.find(Selector.TITLE), this.getTitle());
    var content = this._getContent();
    if (typeof content === 'function') {
      content = content.call(this.element);
    }
    this.setElementContent($tip.find(Selector.CONTENT), content);

    $tip.removeClass(((ClassName.FADE) + " " + (ClassName.SHOW)));
  };

  // Private

  Popover.prototype._getContent = function _getContent () {
    return this.element.getAttribute('data-content') ||
      this.config.content
  };

  Popover.prototype._cleanTipClass = function _cleanTipClass () {
    var $tip = $(this.getTipElement());
    var tabClass = $tip.attr('class').match(BSCLS_PREFIX_REGEX);
    if (tabClass !== null && tabClass.length > 0) {
      $tip.removeClass(tabClass.join(''));
    }
  };

  // Static

  Popover._jQueryInterface = function _jQueryInterface (config) {
    return this.each(function () {
      var data = $(this).data(DATA_KEY);
      var _config = typeof config === 'object' ? config : null;

      if (!data && /dispose|hide/.test(config)) {
        return
      }

      if (!data) {
        data = new Popover(this, _config);
        $(this).data(DATA_KEY, data);
      }

      if (typeof config === 'string') {
        if (typeof data[config] === 'undefined') {
          throw new TypeError(("No method named \"" + config + "\""))
        }
        data[config]();
      }
    })
  };

  Object.defineProperties( Popover, staticAccessors );

  return Popover;
}(Tooltip));

/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 */

$.fn[NAME] = Popover._jQueryInterface;
$.fn[NAME].Constructor = Popover;
$.fn[NAME].noConflict = function () {
  $.fn[NAME] = JQUERY_NO_CONFLICT;
  return Popover._jQueryInterface
};

return Popover;

})));
//# sourceMappingURL=popover.js.map
