/*!
  * Bootstrap scrollspy.js v4.3.1 (https://getbootstrap.com/)
  * Copyright 2011-2019 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
  */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('jquery'), require('./util.js')) :
	typeof define === 'function' && define.amd ? define(['jquery', './util.js'], factory) :
	(global.ScrollSpy = factory(global.jQuery,global.Util));
}(this, (function ($,Util) { 'use strict';

$ = $ && $.hasOwnProperty('default') ? $['default'] : $;
Util = Util && Util.hasOwnProperty('default') ? Util['default'] : Util;

/**
 * --------------------------------------------------------------------------
 * Bootstrap (v4.3.1): scrollspy.js
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * --------------------------------------------------------------------------
 */

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */

var NAME               = 'scrollspy';
var VERSION            = '4.3.1';
var DATA_KEY           = 'bs.scrollspy';
var EVENT_KEY          = "." + DATA_KEY;
var DATA_API_KEY       = '.data-api';
var JQUERY_NO_CONFLICT = $.fn[NAME];

var Default = {
  offset : 10,
  method : 'auto',
  target : ''
};

var DefaultType = {
  offset : 'number',
  method : 'string',
  target : '(string|element)'
};

var Event = {
  ACTIVATE      : ("activate" + EVENT_KEY),
  SCROLL        : ("scroll" + EVENT_KEY),
  LOAD_DATA_API : ("load" + EVENT_KEY + DATA_API_KEY)
};

var ClassName = {
  DROPDOWN_ITEM : 'dropdown-item',
  DROPDOWN_MENU : 'dropdown-menu',
  ACTIVE        : 'active'
};

var Selector = {
  DATA_SPY        : '[data-spy="scroll"]',
  ACTIVE          : '.active',
  NAV_LIST_GROUP  : '.nav, .list-group',
  NAV_LINKS       : '.nav-link',
  NAV_ITEMS       : '.nav-item',
  LIST_ITEMS      : '.list-group-item',
  DROPDOWN        : '.dropdown',
  DROPDOWN_ITEMS  : '.dropdown-item',
  DROPDOWN_TOGGLE : '.dropdown-toggle'
};

var OffsetMethod = {
  OFFSET   : 'offset',
  POSITION : 'position'
};

/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var ScrollSpy = function ScrollSpy(element, config) {
  var this$1 = this;

  this._element     = element;
  this._scrollElement = element.tagName === 'BODY' ? window : element;
  this._config      = this._getConfig(config);
  this._selector    = (this._config.target) + " " + (Selector.NAV_LINKS) + "," +
                        (this._config.target) + " " + (Selector.LIST_ITEMS) + "," +
                        (this._config.target) + " " + (Selector.DROPDOWN_ITEMS);
  this._offsets     = [];
  this._targets     = [];
  this._activeTarget= null;
  this._scrollHeight= 0;

  $(this._scrollElement).on(Event.SCROLL, function (event) { return this$1._process(event); });

  this.refresh();
  this._process();
};

var staticAccessors = { VERSION: { configurable: true },Default: { configurable: true } };

// Getters

staticAccessors.VERSION.get = function () {
  return VERSION
};

staticAccessors.Default.get = function () {
  return Default
};

// Public

ScrollSpy.prototype.refresh = function refresh () {
    var this$1 = this;

  var autoMethod = this._scrollElement === this._scrollElement.window
    ? OffsetMethod.OFFSET : OffsetMethod.POSITION;

  var offsetMethod = this._config.method === 'auto'
    ? autoMethod : this._config.method;

  var offsetBase = offsetMethod === OffsetMethod.POSITION
    ? this._getScrollTop() : 0;

  this._offsets = [];
  this._targets = [];

  this._scrollHeight = this._getScrollHeight();

  var targets = [].slice.call(document.querySelectorAll(this._selector));

  targets
    .map(function (element) {
      var target;
      var targetSelector = Util.getSelectorFromElement(element);

      if (targetSelector) {
        target = document.querySelector(targetSelector);
      }

      if (target) {
        var targetBCR = target.getBoundingClientRect();
        if (targetBCR.width || targetBCR.height) {
          // TODO (fat): remove sketch reliance on jQuery position/offset
          return [
            $(target)[offsetMethod]().top + offsetBase,
            targetSelector
          ]
        }
      }
      return null
    })
    .filter(function (item) { return item; })
    .sort(function (a, b) { return a[0] - b[0]; })
    .forEach(function (item) {
      this$1._offsets.push(item[0]);
      this$1._targets.push(item[1]);
    });
};

ScrollSpy.prototype.dispose = function dispose () {
  $.removeData(this._element, DATA_KEY);
  $(this._scrollElement).off(EVENT_KEY);

  this._element     = null;
  this._scrollElement = null;
  this._config      = null;
  this._selector    = null;
  this._offsets     = null;
  this._targets     = null;
  this._activeTarget= null;
  this._scrollHeight= null;
};

// Private

ScrollSpy.prototype._getConfig = function _getConfig (config) {
  config = Object.assign({}, Default,
    typeof config === 'object' && config ? config : {});

  if (typeof config.target !== 'string') {
    var id = $(config.target).attr('id');
    if (!id) {
      id = Util.getUID(NAME);
      $(config.target).attr('id', id);
    }
    config.target = "#" + id;
  }

  Util.typeCheckConfig(NAME, config, DefaultType);

  return config
};

ScrollSpy.prototype._getScrollTop = function _getScrollTop () {
  return this._scrollElement === window
    ? this._scrollElement.pageYOffset : this._scrollElement.scrollTop
};

ScrollSpy.prototype._getScrollHeight = function _getScrollHeight () {
  return this._scrollElement.scrollHeight || Math.max(
    document.body.scrollHeight,
    document.documentElement.scrollHeight
  )
};

ScrollSpy.prototype._getOffsetHeight = function _getOffsetHeight () {
  return this._scrollElement === window
    ? window.innerHeight : this._scrollElement.getBoundingClientRect().height
};

ScrollSpy.prototype._process = function _process () {
    var this$1 = this;

  var scrollTop  = this._getScrollTop() + this._config.offset;
  var scrollHeight = this._getScrollHeight();
  var maxScroll  = this._config.offset +
    scrollHeight -
    this._getOffsetHeight();

  if (this._scrollHeight !== scrollHeight) {
    this.refresh();
  }

  if (scrollTop >= maxScroll) {
    var target = this._targets[this._targets.length - 1];

    if (this._activeTarget !== target) {
      this._activate(target);
    }
    return
  }

  if (this._activeTarget && scrollTop < this._offsets[0] && this._offsets[0] > 0) {
    this._activeTarget = null;
    this._clear();
    return
  }

  var offsetLength = this._offsets.length;
  for (var i = offsetLength; i--;) {
    var isActiveTarget = this$1._activeTarget !== this$1._targets[i] &&
        scrollTop >= this$1._offsets[i] &&
        (typeof this$1._offsets[i + 1] === 'undefined' ||
            scrollTop < this$1._offsets[i + 1]);

    if (isActiveTarget) {
      this$1._activate(this$1._targets[i]);
    }
  }
};

ScrollSpy.prototype._activate = function _activate (target) {
  this._activeTarget = target;

  this._clear();

  var queries = this._selector
    .split(',')
    .map(function (selector) { return (selector + "[data-target=\"" + target + "\"]," + selector + "[href=\"" + target + "\"]"); });

  var $link = $([].slice.call(document.querySelectorAll(queries.join(','))));

  if ($link.hasClass(ClassName.DROPDOWN_ITEM)) {
    $link.closest(Selector.DROPDOWN).find(Selector.DROPDOWN_TOGGLE).addClass(ClassName.ACTIVE);
    $link.addClass(ClassName.ACTIVE);
  } else {
    // Set triggered link as active
    $link.addClass(ClassName.ACTIVE);
    // Set triggered links parents as active
    // With both <ul> and <nav> markup a parent is the previous sibling of any nav ancestor
    $link.parents(Selector.NAV_LIST_GROUP).prev(((Selector.NAV_LINKS) + ", " + (Selector.LIST_ITEMS))).addClass(ClassName.ACTIVE);
    // Handle special case when .nav-link is inside .nav-item
    $link.parents(Selector.NAV_LIST_GROUP).prev(Selector.NAV_ITEMS).children(Selector.NAV_LINKS).addClass(ClassName.ACTIVE);
  }

  $(this._scrollElement).trigger(Event.ACTIVATE, {
    relatedTarget: target
  });
};

ScrollSpy.prototype._clear = function _clear () {
  [].slice.call(document.querySelectorAll(this._selector))
    .filter(function (node) { return node.classList.contains(ClassName.ACTIVE); })
    .forEach(function (node) { return node.classList.remove(ClassName.ACTIVE); });
};

// Static

ScrollSpy._jQueryInterface = function _jQueryInterface (config) {
  return this.each(function () {
    var data = $(this).data(DATA_KEY);
    var _config = typeof config === 'object' && config;

    if (!data) {
      data = new ScrollSpy(this, _config);
      $(this).data(DATA_KEY, data);
    }

    if (typeof config === 'string') {
      if (typeof data[config] === 'undefined') {
        throw new TypeError(("No method named \"" + config + "\""))
      }
      data[config]();
    }
  })
};

Object.defineProperties( ScrollSpy, staticAccessors );

/**
 * ------------------------------------------------------------------------
 * Data Api implementation
 * ------------------------------------------------------------------------
 */

$(window).on(Event.LOAD_DATA_API, function () {
  var scrollSpys = [].slice.call(document.querySelectorAll(Selector.DATA_SPY));
  var scrollSpysLength = scrollSpys.length;

  for (var i = scrollSpysLength; i--;) {
    var $spy = $(scrollSpys[i]);
    ScrollSpy._jQueryInterface.call($spy, $spy.data());
  }
});

/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 */

$.fn[NAME] = ScrollSpy._jQueryInterface;
$.fn[NAME].Constructor = ScrollSpy;
$.fn[NAME].noConflict = function () {
  $.fn[NAME] = JQUERY_NO_CONFLICT;
  return ScrollSpy._jQueryInterface
};

return ScrollSpy;

})));
//# sourceMappingURL=scrollspy.js.map
