/*!
  * Bootstrap toast.js v4.3.1 (https://getbootstrap.com/)
  * Copyright 2011-2019 The Bootstrap Authors (https://github.com/twbs/bootstrap/graphs/contributors)
  * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
  */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('jquery'), require('./util.js')) :
	typeof define === 'function' && define.amd ? define(['jquery', './util.js'], factory) :
	(global.Toast = factory(global.jQuery,global.Util));
}(this, (function ($,Util) { 'use strict';

$ = $ && $.hasOwnProperty('default') ? $['default'] : $;
Util = Util && Util.hasOwnProperty('default') ? Util['default'] : Util;

/**
 * --------------------------------------------------------------------------
 * Bootstrap (v4.3.1): toast.js
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * --------------------------------------------------------------------------
 */

/**
 * ------------------------------------------------------------------------
 * Constants
 * ------------------------------------------------------------------------
 */

var NAME               = 'toast';
var VERSION            = '4.3.1';
var DATA_KEY           = 'bs.toast';
var EVENT_KEY          = "." + DATA_KEY;
var JQUERY_NO_CONFLICT = $.fn[NAME];

var Event = {
  CLICK_DISMISS : ("click.dismiss" + EVENT_KEY),
  HIDE          : ("hide" + EVENT_KEY),
  HIDDEN        : ("hidden" + EVENT_KEY),
  SHOW          : ("show" + EVENT_KEY),
  SHOWN         : ("shown" + EVENT_KEY)
};

var ClassName = {
  FADE    : 'fade',
  HIDE    : 'hide',
  SHOW    : 'show',
  SHOWING : 'showing'
};

var DefaultType = {
  animation : 'boolean',
  autohide  : 'boolean',
  delay     : 'number'
};

var Default = {
  animation : true,
  autohide  : true,
  delay     : 500
};

var Selector = {
  DATA_DISMISS : '[data-dismiss="toast"]'
};

/**
 * ------------------------------------------------------------------------
 * Class Definition
 * ------------------------------------------------------------------------
 */

var Toast = function Toast(element, config) {
  this._element = element;
  this._config= this._getConfig(config);
  this._timeout = null;
  this._setListeners();
};

var staticAccessors = { VERSION: { configurable: true },DefaultType: { configurable: true },Default: { configurable: true } };

// Getters

staticAccessors.VERSION.get = function () {
  return VERSION
};

staticAccessors.DefaultType.get = function () {
  return DefaultType
};

staticAccessors.Default.get = function () {
  return Default
};

// Public

Toast.prototype.show = function show () {
    var this$1 = this;

  $(this._element).trigger(Event.SHOW);

  if (this._config.animation) {
    this._element.classList.add(ClassName.FADE);
  }

  var complete = function () {
    this$1._element.classList.remove(ClassName.SHOWING);
    this$1._element.classList.add(ClassName.SHOW);

    $(this$1._element).trigger(Event.SHOWN);

    if (this$1._config.autohide) {
      this$1.hide();
    }
  };

  this._element.classList.remove(ClassName.HIDE);
  this._element.classList.add(ClassName.SHOWING);
  if (this._config.animation) {
    var transitionDuration = Util.getTransitionDurationFromElement(this._element);

    $(this._element)
      .one(Util.TRANSITION_END, complete)
      .emulateTransitionEnd(transitionDuration);
  } else {
    complete();
  }
};

Toast.prototype.hide = function hide (withoutTimeout) {
    var this$1 = this;

  if (!this._element.classList.contains(ClassName.SHOW)) {
    return
  }

  $(this._element).trigger(Event.HIDE);

  if (withoutTimeout) {
    this._close();
  } else {
    this._timeout = setTimeout(function () {
      this$1._close();
    }, this._config.delay);
  }
};

Toast.prototype.dispose = function dispose () {
  clearTimeout(this._timeout);
  this._timeout = null;

  if (this._element.classList.contains(ClassName.SHOW)) {
    this._element.classList.remove(ClassName.SHOW);
  }

  $(this._element).off(Event.CLICK_DISMISS);

  $.removeData(this._element, DATA_KEY);
  this._element = null;
  this._config= null;
};

// Private

Toast.prototype._getConfig = function _getConfig (config) {
  config = Object.assign({}, Default,
    $(this._element).data(),
    typeof config === 'object' && config ? config : {});

  Util.typeCheckConfig(
    NAME,
    config,
    this.constructor.DefaultType
  );

  return config
};

Toast.prototype._setListeners = function _setListeners () {
    var this$1 = this;

  $(this._element).on(
    Event.CLICK_DISMISS,
    Selector.DATA_DISMISS,
    function () { return this$1.hide(true); }
  );
};

Toast.prototype._close = function _close () {
    var this$1 = this;

  var complete = function () {
    this$1._element.classList.add(ClassName.HIDE);
    $(this$1._element).trigger(Event.HIDDEN);
  };

  this._element.classList.remove(ClassName.SHOW);
  if (this._config.animation) {
    var transitionDuration = Util.getTransitionDurationFromElement(this._element);

    $(this._element)
      .one(Util.TRANSITION_END, complete)
      .emulateTransitionEnd(transitionDuration);
  } else {
    complete();
  }
};

// Static

Toast._jQueryInterface = function _jQueryInterface (config) {
  return this.each(function () {
    var $element = $(this);
    var data     = $element.data(DATA_KEY);
    var _config= typeof config === 'object' && config;

    if (!data) {
      data = new Toast(this, _config);
      $element.data(DATA_KEY, data);
    }

    if (typeof config === 'string') {
      if (typeof data[config] === 'undefined') {
        throw new TypeError(("No method named \"" + config + "\""))
      }

      data[config](this);
    }
  })
};

Object.defineProperties( Toast, staticAccessors );

/**
 * ------------------------------------------------------------------------
 * jQuery
 * ------------------------------------------------------------------------
 */

$.fn[NAME]             = Toast._jQueryInterface;
$.fn[NAME].Constructor = Toast;
$.fn[NAME].noConflict  = function () {
  $.fn[NAME] = JQUERY_NO_CONFLICT;
  return Toast._jQueryInterface
};

return Toast;

})));
//# sourceMappingURL=toast.js.map
